unit hiAdjustControls;

interface

uses Windows, Messages, Kol, Share, Debug, Win;

const
  DTM_FIRST            = $1000;
  DTM_GETSYSTEMTIME    = DTM_FIRST + 1;
  DTM_SETSYSTEMTIME    = DTM_FIRST + 2;
  GDT_VALID            = 0;

  AlignName: array [0..5] of string = (
    'none','left','top','rigth','bottom','client' );
     
  PropNames: array[0..22] of string = (
    'left','top','width','height','align','color',
    'text','caption','strings','visible','enabled','checked','selected',
    'select','fontname','fontcolor','fontsize', 'fontstyle','fontcharset',
    'redraw', 'sendtoback', 'bringtofront', 'setfocus' );

  ColorValues: array [0..46] of TColor = (
    clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,clGray,
    clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite,
    clMoneyGreen,clSkyBlue,clCream,clMedGray,clActiveBorder,clActiveCaption,
    clAppWorkSpace,clBackground,clBtnFace,clBtnHighlight,clBtnShadow,    
    clBtnText,clCaptionText,clDefault,clGrayText,clHighlight,clHighlightText,
    clInactiveBorder,clInactiveCaption,clInactiveCaptionText,clInfoBk,
    clInfoText,clMenu,clMenuText,clNone,clScrollBar,cl3DDkShadow,
    cl3DLight,clWindow,clWindowFrame,clWindowText );

  ColorNames: array [0..46] of string = (
    'black','maroon','green','olive','navy','purple','teal','gray',
    'silver','red','lime','yellow','blue','fuchsia','aqua','white',
    'moneygreen','skyblue','cream','medgray','activeborder','activecaption',
    'appworkspace','background','btnface','btnhighlight','btnshadow',    
    'btntext','captiontext','default','graytext','highlight','highlighttext',
    'inactiveborder','inactivecaption','inactivecaptiontext','infobk',
    'infotext','menu','menutext','none','scrollbar','3ddkshadow',
    '3dlight','window','windowframe','windowtext' );

type
  THIAdjustControls = class(TDebug)
   private
    FListSet,
    FListIncludeName,
    FListExcludeName,
    FListIncludeProp,
    FListExcludeProp,
    FListProp: PStrList;

    procedure SetParameters(const Value: string);
    procedure SetIncludeNameList(const Value: string);
    procedure SetExcludeNameList(const Value: string);
    procedure SetIncludePropList(const Value: string);
    procedure SetExcludePropList(const Value: string);
    function GetIntColor(const Value: string): integer;    
    function GetStrColor(const Value: integer): string;
    function isTrueName(Name: string): boolean;
    function isTrueProp(Prop: string): boolean;
   public
    _prop_FormatDate,
    _prop_Parameters,
    _prop_IncludeNameList,
    _prop_ExcludeNameList,
    _prop_IncludePropList,
    _prop_ExcludePropList: string;
    _data_NameControl,
    _data_Parameters,
    _data_IncludeNameList,
    _data_ExcludeNameList,
    _data_IncludePropList,
    _data_ExcludePropList,
    _event_onSet,
    _event_onEnumNameControls,
    _event_onEndEnumNameControls,
    _event_onEnumParameters,
    _event_onEndEnumParameters: THI_Event;
    
    constructor Create;
    destructor Destroy; override;
    procedure _work_doSet(var _Data: TData; Index: word);
    procedure _work_doEnumNameControls(var _Data: TData; Index: word);
    procedure _work_doEnumParameters(var _Data: TData; Index: word);
  end;
    
implementation

uses hiStr_Enum;

constructor THIAdjustControls.Create;
var
  i: integer;
begin
   inherited Create;
   FListSet := NewStrList;
   FListProp := NewStrList;
   FListIncludeName := NewStrList;
   FListExcludeName := NewStrList; 
   FListIncludeProp := NewStrList;
   FListExcludeProp := NewStrList;
   for i := 0 to High(PropNames) do 
     FListProp.Add(PropNames[i]);
end;

destructor THIAdjustControls.Destroy;
begin
   FListSet.Free;
   FListProp.Free;
   FListIncludeName.Free;
   FListExcludeName.Free;
   FListIncludeProp.Free;
   FListExcludeProp.Free;
   inherited;
end;

function THIAdjustControls.GetIntColor;
var
  i: integer;
begin
  for i := 0 to High(ColorNames) do
    if Value = ColorNames[i] then
    begin
      Result := ColorValues[i];
      exit;
    end;
  Result := str2int(Value);
end;

function THIAdjustControls.GetStrColor;
var
  i: integer;
begin
  for i := 0 to High(ColorValues) do
    if Color2RGB(Value) = Color2RGB(ColorValues[i]) then
    begin
      Result := ColorNames[i];
      exit;
    end;
  Result := int2str(Value);
end;

function THIAdjustControls.isTrueName(Name: string): boolean;
begin
  Result := ((FListIncludeName.IndexOf(Name) <> -1) or
            (FListIncludeName.IndexOf('all') <> -1)) and
            (FListExcludeName.IndexOf(Name) = -1);
end;

function THIAdjustControls.isTrueProp(Prop: string): boolean;
begin
  Result := ((FListIncludeProp.IndexOf(prop) <> -1) or
            (FListIncludeProp.IndexOf('all') <> -1)) and
            (FListExcludeProp.IndexOf(prop) = -1);
end;

//------------------------------- Set Lists ------------------------------------

procedure THIAdjustControls.SetParameters;
begin
  if Value <> '' then FListSet.SetText(Value, false)
end;

procedure THIAdjustControls.SetIncludeNameList;
begin
  if Value <> '' then FListIncludeName.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetExcludeNameList;
begin
  if Value <> '' then FListExcludeName.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetIncludePropList;
begin
  if Value <> '' then FListIncludeProp.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetExcludePropList;
begin
  if Value <> '' then FListExcludeProp.SetText(LowerCase(Value), false);
end;

//------------------------------------------------------------------------------

procedure THIAdjustControls._work_doSet;
var
  tmpControl: PControl;
  i, j, val, _itc: integer;
  dt: TDateTime;
  st: TSystemTime;
  _str, str, _ntc, s_name, _s_name, s_prop: string;
  aName: array [0..255] of Char;
  
   procedure change(ind: integer);
   var
     k: integer;
   begin
     GetClassName(tmpControl.Handle, aName, 256);
     with tmpControl{$ifndef F_P}^{$endif} do
     case ind of
       0: Left := str2int(str);
       1: Top := str2int(str);
       2: Width := str2int(str);
       3: Height := str2int(str);
       4: begin
            if (LowerCase(str) = AlignName[0]) or (str ='0') then Align := caNone   else // none
            if (LowerCase(str) = AlignName[1]) or (str ='1') then Align := caLeft   else // left
            if (LowerCase(str) = AlignName[2]) or (str ='2') then Align := caTop    else // top         
            if (LowerCase(str) = AlignName[3]) or (str ='3') then Align := caRight  else // rigth 
            if (LowerCase(str) = AlignName[4]) or (str ='4') then Align := caBottom else // bottom 
            if (LowerCase(str) = AlignName[5]) or (str ='5') then Align := caClient;     // client 
            Invalidate;
          end;
       5: Color := GetIntColor(LowerCase(str)); 
     6,7: begin // text = caption
            if isForm then
              SetWindowText(Handle, @str[1])
            else if aName = 'obj_COMBOBOX' then Text := str
            else if (aName = 'obj_SysDateTimePick32') then 
            begin
              if  length(str) > 0 then
              begin
                if (str[1] = '@') then
                begin
                  Replace(str, '@', '0');
                  dt := str2double(str); 
                end
                else
                  dt := Str2DateTimeFmt(_prop_FormatDate, str);              
                DateTime2SystemTime(dt, st);
              end
              else
                GetLocalTime(st);  
            end;
            Perform(DTM_SETSYSTEMTIME, GDT_VALID, Longint(@st));
          end;
       8: begin // strings
            if (aName = 'obj_LISTBOX') or (aName = 'obj_COMBOBOX') then
            begin
              Clear;
              while str <> '' do
              begin
                _str := fparse(str,'|');
                Add(fparse(_str,'#'));
                ItemData[Count - 1] := str2int(_str); 
              end
            end
            else if (aName = 'obj_SysTabControl32') then
            begin
              for k := Count - 1 downto 0 do TC_Delete(k);
              while str <> '' do
              begin
                _str := fparse(str,'|');
                _ntc := fparse(_str,'#');
                if _str = '' then
                  _itc := Count
                else
                  _itc := str2int(_str);
                TC_Insert(Count, _ntc, _itc);
              end;  
            end
            else if (aName = 'obj_EDIT') or (aName = 'obj_RichEdit20A') or
                    (aName = 'obj_BUTTON') or (aName = 'obj_STATIC')  then
            begin 
              Replace(str, '|', #13#10);
              Text := str;
            end 
          end;
       9: Visible := boolean(ord(str2int(str)));
      10: Enabled := boolean(ord(str2int(str)));
   11,12: Checked := boolean(ord(str2int(str))); // checked = selected
      13: begin // select -  
            val := str2int(str);
            if (val < 0) or (val >= Count) then val := -1;
            if not ((val < 0) and (aName = 'obj_COMBOBOX')) then 
              CurIndex := val;
          end;
      14: Font.FontName := str;
      15: Font.Color := GetIntColor(LowerCase(str));
      16: Font.FontHeight := _hi_SizeFnt(str2int(str));
      17: begin
            Font.FontStyle := [];
            _str := LowerCase(str);
            for k := 1 to length(_str) do
            begin
              if _str[k] = 'b' then Font.FontStyle := Font.FontStyle + [fsBold] else
              if _str[k] = 'i' then Font.FontStyle := Font.FontStyle + [fsItalic] else
              if _str[k] = 'u' then Font.FontStyle := Font.FontStyle + [fsUnderline] else
              if _str[k] = 's' then Font.FontStyle := Font.FontStyle + [fsStrikeOut];
            end;
          end;
      18: Font.FontCharset := str2int(str);
      19: if str = 'true' then
              InvalidateRect(Handle, nil, true)
            else
              InvalidateRect(Handle, nil, false);
      20: tmpControl.SendToBack;
      21: tmpControl.BringToFront;
      22: Focused := true;               
     end;
   end;

begin
  SetParameters(ReadString(_Data, _data_Parameters, _prop_Parameters));
  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));
  SetIncludePropList(ReadString(_Data, _data_IncludePropList, _prop_IncludePropList));
  SetExcludePropList(ReadString(_Data, _data_ExcludePropList, _prop_ExcludePropList));

  for i := 0 to ListOfNameControls.Count - 1 do
  begin
    tmpControl := PControl(ListOfNameControls.Objects[i]);
    with tmpControl{$ifndef F_P}^{$endif}  do
    if isControl or isForm then
    begin
      s_name := '';
      for j := 0 to FListSet.Count - 1 do
      begin
        str := FListSet.Items[j];
        if str <> '' then
        begin
          _s_name := LowerCase(fparse(str,'='));
          if (_s_name <> s_name) and (_s_name <> '*') then s_name := _s_name; 
          if ((s_name = LowerCase(ListOfNameControls.Items[i])) or (s_name = 'all')) and isTrueName(s_name) then
          begin
            s_prop := LowerCase(fparse(str,'='));
            if isTrueProp(s_prop) then change(FListProp.IndexOf(s_prop));
          end;
        end;
      end;
    end;
  end;
  _hi_OnEvent(_event_onSet);
end;

procedure THIAdjustControls._work_doEnumNameControls;
var
  i: integer;
begin
  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));

  for i := 0 to ListOfNameControls.Count - 1 do
    if isTrueName(ListOfNameControls.Items[i]) then
      _hi_onEvent(_event_onEnumNameControls, ListOfNameControls.Items[i]);
  _hi_onEvent(_event_onEndEnumNameControls);      
end;

procedure THIAdjustControls._work_doEnumParameters;
var
  r: real;
  i: integer;
  dt: TDateTime;
  st: TSystemTime;  
  sFontSize: integer;
  tmpControl: PControl;
  name, sFontStyle, sText: string;
  dtname, dtprop, dtval: TData;
  aName: array [0..255] of Char;  
  
  procedure outparameters(prop, value: string);
  begin
    if isTrueProp(prop) then
    begin   
      dtString(dtprop, prop);
      dtString(dtval, value);
      dtval.ldata := @dtprop; 
      _hi_onEvent_(_event_onEnumParameters, dtname);
    end;  
  end;

begin
  name := LowerCase(ReadString(_Data, _data_NameControl));
  if name = '' then exit;

  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));
  SetIncludePropList(ReadString(_Data, _data_IncludePropList, _prop_IncludePropList));
  SetExcludePropList(ReadString(_Data, _data_ExcludePropList, _prop_ExcludePropList));

  if not isTrueName(name) then exit;

  tmpControl := PControl(ListOfNameControls.Objects[ListOfNameControls.IndexOf(name)]);
  if not (tmpControl.isControl or tmpControl.isForm) then exit;

  dtString(dtname, name); 
  dtname.ldata := @dtval;
  with tmpControl{$ifndef F_P}^{$endif} do
  begin
    outparameters(PropNames[0], int2str(Left));              // left
    outparameters(PropNames[1], int2str(Top));               // top
    outparameters(PropNames[2], int2str(Width));             // width
    outparameters(PropNames[3], int2str(Height));            // height
    outparameters(PropNames[4], AlignName[ord(Align)]);      // align
    outparameters(PropNames[5], GetStrColor(Color));         // color
    GetClassName(Handle, aName, 256);

    GetClassName(Handle, aName, 256);
    if (aName = 'obj_LISTBOX') or (aName = 'obj_COMBOBOX') then
    begin
      sText := '';
      for i := 0 to Count - 1 do 
        sText := sText + Items[i] + '#' + int2str(ItemData[i]) + '|';
    end
    else if (aName = 'obj_SysTabControl32') then
    begin
      sText := '';
      for i := 0 to Count - 1 do 
        sText := sText + TC_Items[i] + '#' + int2str(TC_Images[i]) + '|';
    end
    else if (aName = 'obj_EDIT') or (aName = 'obj_RichEdit20A') or
       (aName = 'obj_BUTTON') or (aName = 'obj_STATIC')  then
    begin 
      sText := Text;
      Replace(sText, #13#10, '|');
    end 
    else
      sText := '';      
    outparameters(PropNames[8], sText);                      // strings

    if isForm or (aName = 'obj_COMBOBOX') then
      sText := Text
    else if (aName = 'obj_SysDateTimePick32') then 
    begin
      Perform(DTM_GETSYSTEMTIME, 0, Longint(@st));
      SystemTime2DateTime(st, dt);
      sText := '@' + double2str(dt);
    end
    else
      sText := '';      
    outparameters(PropNames[6], sText);                      // text

    outparameters(PropNames[9], int2str(ord(Visible)));      // visible
    outparameters(PropNames[10], int2str(ord(Enabled)));     // enabled
    outparameters(PropNames[11], int2str(ord(Checked)));     // checked                    
    outparameters(PropNames[12], int2str(ord(Checked)));     // selected
    outparameters(PropNames[13], int2str(CurIndex));         // select    
    outparameters(PropNames[14], Font.FontName);             // fontname  
    outparameters(PropNames[15], GetStrColor(Font.Color));   // fontcolor
    r := ((Font.FontHeight * -72) - 36) / ScreenDPI;
    sFontSize := Trunc(r);
    if Frac(r) > 0 then Inc(sFontSize);
    outparameters(PropNames[16], int2str(sFontSize));        // fontsize
    sFontStyle := '';
    if fsBold in Font.FontStyle then  
      sFontStyle := sFontStyle + 'b';
    if fsItalic in Font.FontStyle then
      sFontStyle := sFontStyle + 'i';     
    if fsUnderline in Font.FontStyle then
      sFontStyle := sFontStyle + 'u';     
    if fsStrikeOut in Font.FontStyle then
      sFontStyle := sFontStyle + 's';     
    outparameters(PropNames[17], sFontStyle);                // fontstyle     
    outparameters(PropNames[18], int2str(Font.FontCharset)); // fontcharset
  end;
  _hi_onEvent(_event_onEndEnumParameters);   
end;

end.